package com.tenlionsoft.baselib.utils

import android.app.ActivityManager
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.net.ConnectivityManager
import android.net.Network
import android.net.NetworkCapabilities
import android.net.Uri
import android.os.Build
import androidx.core.content.FileProvider
import com.tenlionsoft.baselib.base.App
import java.io.File
import java.io.IOException
import java.util.UUID


class AppUtils {
    companion object {
        fun getAppVersionName(): String {
            return getAppVersionName(App.context.packageName)
        }

        private fun getAppVersionName(pkgName: String?): String {
            if (pkgName.isNullOrEmpty()) return ""
            try {
                val pm = App.context.packageManager
                val pi = pm.getPackageInfo(pkgName, 0)
                return pi?.versionName
                    ?: ""
            } catch (e: PackageManager.NameNotFoundException) {
                e.printStackTrace();
                return ""
            }
        }

        fun getAppVersionCode(): Long {
            return getAppVersionCode(App.context.packageName)
        }


        fun isRunningService(ctx: Context, className: String): Boolean {
            var isRunning = false
            val activityManager = ctx
                .getSystemService(Context.ACTIVITY_SERVICE) as ActivityManager
            val servicesList = activityManager
                .getRunningServices(Int.MAX_VALUE)
            val l: Iterator<ActivityManager.RunningServiceInfo> = servicesList.iterator()
            while (l.hasNext()) {
                val si = l.next()
                if (className == si.service.className) {
                    isRunning = true
                }

                if (className == si.process) {
                    isRunning = true
                }
            }
            return isRunning
        }

        private fun getAppVersionCode(pkgName: String?): Long {
            if (pkgName.isNullOrEmpty()) return -1
            try {
                val pm = App.context.packageManager
                val pi = pm.getPackageInfo(pkgName, 0);
                if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
                    return pi?.longVersionCode
                        ?: -1
                } else {
                    return (pi?.versionCode
                        ?: -1).toLong()
                }
            } catch (e: PackageManager.NameNotFoundException) {
                e.printStackTrace()
                return -1
            }
        }

        fun getDeviceId(): String {
            val filePath: String =
                App.context.getExternalFilesDir("Cache")
                    ?.absolutePath + File.separator + "identity.txt"
            val file = File(filePath)
            var uuid: String? = ""
            if (!file.exists()) {
                try {
                    val newFile = file.createNewFile()
                    uuid = UUID.randomUUID()
                        .toString()
                    FileIOUtils.writeFileFromString(file, uuid)
                } catch (e: IOException) {
                    e.printStackTrace()
                }
            } else {
                uuid = FileIOUtils.readFile2String(file)
            }
            return uuid!!
        }

        /**
         * 根据版本号大小判断是否需要更新
         *
         * @param context
         * @param versionCode 服务端传回来的版本号
         * @return
         */
        fun checkcode(versionCode: String): Boolean {
            return getAppVersionCode() < versionCode.toInt()
        }

        /**
         * 安装apk
         *
         * @param context context
         * @param apkFile 安装文件
         */
        fun installAPk(context: Context, apkFile: File) {
            val installAPKIntent: Intent = getApkInStallIntent(context, apkFile)
            context.startActivity(installAPKIntent)
        }

        private fun getApkInStallIntent(context: Context, apkFile: File): Intent {
            val intent = Intent(Intent.ACTION_VIEW)
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)

            if (Build.VERSION.SDK_INT > Build.VERSION_CODES.M) {
                val uri = FileProvider.getUriForFile(
                    context,
                    ProiderUtils.getFileProviderName(context),
                    apkFile
                )
                intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
                intent.setDataAndType(uri, "application/vnd.android.package-archive")
            } else {
                val uri = getApkUri(apkFile)
                intent.setDataAndType(uri, "application/vnd.android.package-archive")
            }
            return intent
        }

        /**
         * 获取安装文件的Uri
         *
         * @param apkFile 安装文件
         * @return Uri
         */
        private fun getApkUri(apkFile: File): Uri {
            //如果没有设置 SDCard 写权限，或者没有 SDCard,apk 文件保存在内存中，需要授予权限才能安装
            try {
                val command = arrayOf("chmod", "777", apkFile.toString())
                val builder = ProcessBuilder(*command)
                builder.start()
            } catch (ignored: IOException) {
            }
            return Uri.fromFile(apkFile)
        }

        //检查网络是否连接
        private fun isNetworkConnected(context: Context?): Boolean {
            if (context != null) {
                val cm =
                    context.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
                val activeNetwork: Network = cm.activeNetwork
                    ?: return false
                val capabilities = cm.getNetworkCapabilities(activeNetwork)
                return capabilities != null && capabilities.hasCapability(NetworkCapabilities.NET_CAPABILITY_INTERNET)
            } else {
                return false;
            }
        }
    }
}